import { Control, InputMode, Language } from './enums';
import { Ped, Player, Prop, Vehicle } from './models';
export declare abstract class Game {
    /**
     * Calculate the Jenkins One At A Time (joaat) has from the given string.
     *
     * @param input The input string to calculate the hash
     */
    static generateHash(input: string): number;
    /**
     * Gets the game language
     */
    static readonly Language: Language;
    /**
     * Gets how many milliseconds the game has been open this session
     */
    static readonly GameTime: number;
    /**
     * Sets the time scale of the Game.
     *
     * @param time The time scale, only accepts values between 0.0 and 1.0
     */
    static TimeScale: number;
    /**
     * Gets the total amount of frames rendered in this session
     */
    static readonly FrameCount: number;
    /**
     * Gets the current frame rate per second
     */
    static readonly FPS: number;
    /**
     * Gets the time it currently takes to render a frame, in seconds;
     */
    static readonly LastFrameTime: number;
    static readonly Player: Player;
    static readonly PlayerPed: Ped;
    static readonly MaxPlayers: number;
    static playerList(): IterableIterator<Player>;
    /**
     * Get whether PvP is enabled.
     * @returns True if enabled.
     */
    /**
    * Set whether PvP is enabled.
    */
    static PlayerVersusPlayer: boolean;
    static MaxWantedLevel: number;
    static WantedMultiplier: number;
    static ShowPoliceBlipsOnRadar: boolean;
    static Nightvision: boolean;
    static ThermalVision: boolean;
    static IsMissionActive: boolean;
    static IsRandomEventActive: boolean;
    static readonly IsCutsceneActive: boolean;
    static readonly IsWaypointActive: boolean;
    static IsPaused: boolean;
    static readonly IsLoading: boolean;
    static readonly CurrentInputMode: InputMode;
    static isControlPressed(index: number, control: Control): boolean;
    static isControlJustPressed(index: number, control: Control): boolean;
    static isDisabledControlJustPressed(index: number, control: Control): boolean;
    static isControlReleased(index: number, control: Control): boolean;
    static isControlJustReleased(index: number, control: Control): boolean;
    static entityFromHandle(handle: number): Ped | Vehicle | Prop | undefined;
    static playSound(soundFile: string, soundSet: string): void;
    static playMusic(musicFile?: string): void;
    static stopMusic(musicFile?: string): void;
    protected static cachedPlayer: Player;
}
