"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Audio_1 = require("./Audio");
const enums_1 = require("./enums");
const models_1 = require("./models");
class Game {
    /**
     * Calculate the Jenkins One At A Time (joaat) has from the given string.
     *
     * @param input The input string to calculate the hash
     */
    static generateHash(input) {
        if (typeof input === 'undefined') {
            return 0;
        }
        return GetHashKey(input);
    }
    /**
     * Gets the game language
     */
    static get Language() {
        return GetUiLanguageId();
    }
    /**
     * Gets how many milliseconds the game has been open this session
     */
    static get GameTime() {
        return GetGameTimer();
    }
    /**
     * Sets the time scale of the Game.
     *
     * @param time The time scale, only accepts values between 0.0 and 1.0
     */
    static set TimeScale(time) {
        SetTimeScale(time <= 1 && time >= 0 ? time : 1);
    }
    /**
     * Gets the total amount of frames rendered in this session
     */
    static get FrameCount() {
        return GetFrameCount();
    }
    /**
     * Gets the current frame rate per second
     */
    static get FPS() {
        return 1 / this.LastFrameTime;
    }
    /**
     * Gets the time it currently takes to render a frame, in seconds;
     */
    static get LastFrameTime() {
        return GetFrameTime();
    }
    static get Player() {
        const handle = PlayerId();
        if (typeof this.cachedPlayer === 'undefined' || handle !== this.cachedPlayer.Handle) {
            this.cachedPlayer = new models_1.Player(handle);
        }
        return this.cachedPlayer;
    }
    static get PlayerPed() {
        return this.Player.Character;
    }
    static get MaxPlayers() {
        return GetConvarInt('sv_maxclients', 0); // not replicated
    }
    static *playerList() {
        const players = [];
        for (let i = 0; i < this.MaxPlayers; i += 1) {
            if (NetworkIsPlayerActive(i)) {
                yield new models_1.Player(i);
            }
        }
    }
    /**
     * Get whether PvP is enabled.
     * @returns True if enabled.
     */
    static get PlayerVersusPlayer() {
        return this.Player.PvPEnabled;
    }
    /**
     * Set whether PvP is enabled.
     */
    static set PlayerVersusPlayer(value) {
        this.Player.PvPEnabled = value;
    }
    static get MaxWantedLevel() {
        return GetMaxWantedLevel();
    }
    static set MaxWantedLevel(value) {
        if (value < 0) {
            value = 0;
        }
        else if (value > 5) {
            value = 5;
        }
        SetMaxWantedLevel(value);
    }
    static set WantedMultiplier(value) {
        SetWantedLevelMultiplier(value);
    }
    static set ShowPoliceBlipsOnRadar(toggle) {
        SetPoliceRadarBlips(toggle);
    }
    static get Nightvision() {
        return !!IsNightvisionActive();
    }
    static set Nightvision(toggle) {
        SetNightvision(toggle);
    }
    static get ThermalVision() {
        return !!IsSeethroughActive();
    }
    static set ThermalVision(toggle) {
        SetSeethrough(toggle);
    }
    static get IsMissionActive() {
        return !!GetMissionFlag();
    }
    static set IsMissionActive(toggle) {
        SetMissionFlag(toggle);
    }
    static get IsRandomEventActive() {
        return GetRandomEventFlag() === 1;
    }
    static set IsRandomEventActive(toggle) {
        SetRandomEventFlag(toggle ? 1 : 0);
    }
    static get IsCutsceneActive() {
        return !!IsCutsceneActive();
    }
    static get IsWaypointActive() {
        return !!IsWaypointActive();
    }
    static get IsPaused() {
        return !!IsPauseMenuActive();
    }
    static set IsPaused(toggle) {
        SetPauseMenuActive(toggle);
    }
    static get IsLoading() {
        return !!GetIsLoadingScreenActive();
    }
    static get CurrentInputMode() {
        return IsInputDisabled(2) ? enums_1.InputMode.MouseAndKeyboard : enums_1.InputMode.GamePad;
    }
    static isControlPressed(index, control) {
        return !!IsControlPressed(index, Number(control));
    }
    static isControlJustPressed(index, control) {
        return !!IsControlJustPressed(index, Number(control));
    }
    static isDisabledControlJustPressed(index, control) {
        return !!IsDisabledControlJustPressed(index, Number(control));
    }
    static isControlReleased(index, control) {
        return !!IsControlReleased(index, Number(control));
    }
    static isControlJustReleased(index, control) {
        return !!IsControlJustReleased(index, Number(control));
    }
    static entityFromHandle(handle) {
        switch (GetEntityType(handle)) {
            case 1:
                return new models_1.Ped(handle);
            case 2:
                return new models_1.Vehicle(handle);
            case 3:
                return new models_1.Prop(handle);
        }
        return null;
    }
    static playSound(soundFile, soundSet) {
        Audio_1.Audio.playSound(soundFile, soundSet);
    }
    static playMusic(musicFile) {
        Audio_1.Audio.playMusic(musicFile);
    }
    static stopMusic(musicFile) {
        Audio_1.Audio.stopMusic(musicFile);
    }
}
exports.Game = Game;
