"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const enums_1 = require("./enums");
const utils_1 = require("./utils");
class ParticleEffectAsset {
    get Asset() {
        return this.assetName;
    }
    constructor(assetName) {
        this.assetName = assetName;
    }
    get AssetName() {
        return this.assetName;
    }
    get IsLoaded() {
        return !!HasNamedPtfxAssetLoaded(this.assetName);
    }
    startNonLoopedAtCoord(effectName, pos, rot = new utils_1.Vector3(0, 0, 0), scale = 1.0, invertAxis = { flags: enums_1.InvertAxisFlags.None }) {
        if (!this.setNextCall()) {
            return false;
        }
        const invertAxisFlags = invertAxis.flags;
        SetPtfxAssetNextCall(this.assetName);
        return (StartParticleFxLoopedAtCoord(effectName, pos.x, pos.y, pos.z, rot.x, rot.y, rot.z, scale, !!(invertAxisFlags & enums_1.InvertAxisFlags.X), !!(invertAxisFlags & enums_1.InvertAxisFlags.Y), !!(invertAxisFlags & enums_1.InvertAxisFlags.Z), false) > 0);
    }
    startNonLoopedOnEntity(effectName, entity, off = new utils_1.Vector3(0, 0, 0), rot = new utils_1.Vector3(0, 0, 0), scale = 1.0, invertAxis = { flags: enums_1.InvertAxisFlags.None }) {
        if (!this.setNextCall()) {
            return false;
        }
        const invertAxisFlags = invertAxis.flags;
        SetPtfxAssetNextCall(this.assetName);
        return !!StartParticleFxLoopedOnEntity(effectName, entity.Handle, off.x, off.y, off.z, rot.x, rot.y, rot.z, scale, !!(invertAxisFlags & enums_1.InvertAxisFlags.X), !!(invertAxisFlags & enums_1.InvertAxisFlags.Y), !!(invertAxisFlags & enums_1.InvertAxisFlags.Z));
    }
    request(timeout) {
        return new Promise((resolve) => {
            if (!this.IsLoaded) {
                RequestNamedPtfxAsset(this.assetName);
                const start = GetGameTimer();
                const interval = setInterval(() => {
                    if (this.IsLoaded || GetGameTimer() - start >= timeout) {
                        clearInterval(interval);
                        resolve(this.IsLoaded);
                    }
                    // tslint:disable-next-line: align
                }, 0);
            }
            else {
                resolve(this.IsLoaded);
            }
        });
    }
    markAsNoLongerNeeded() {
        RemoveNamedPtfxAsset(this.assetName);
    }
    toString() {
        return this.assetName;
    }
    setNextCall() {
        if (!this.IsLoaded) {
            RequestNamedPtfxAsset(this.assetName);
        }
        else {
            SetPtfxAssetNextCall(this.assetName);
            return true;
        }
        return false;
    }
}
exports.ParticleEffectAsset = ParticleEffectAsset;
