"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const _1 = require("./");
const Blip_1 = require("./Blip");
const Camera_1 = require("./Camera");
const enums_1 = require("./enums");
const hashes_1 = require("./hashes");
const models_1 = require("./models");
const Raycast_1 = require("./Raycast");
const utils_1 = require("./utils");
/**
 * Class with common world manipulations.
 *
 * This class includes methods for creating entities and common world rendering.
 */
class World {
    static get RenderingCamera() {
        return new Camera_1.Camera(GetRenderingCam());
    }
    static set RenderingCamera(value) {
        if (value === null) {
            RenderScriptCams(false, false, 3000, true, false);
        }
        else {
            value.IsActive = true;
            RenderScriptCams(true, false, 3000, true, false);
        }
    }
    static get CurrentDate() {
        const year = GetClockYear();
        const month = GetClockMonth();
        const day = GetClockDayOfMonth();
        const hour = GetClockHours();
        const minutes = GetClockMinutes();
        const seconds = GetClockSeconds();
        return new Date(year, month, day, hour, minutes, seconds);
    }
    static set CurrentDate(date) {
        SetClockDate(date.getDate(), date.getMonth(), date.getFullYear());
        SetClockTime(date.getHours(), date.getMinutes(), date.getSeconds());
    }
    static set Blackout(value) {
        SetBlackout(value);
    }
    static get CloudHat() {
        return this.currentCloudHat;
    }
    static set CloudHat(value) {
        this.currentCloudHat = value;
        if (this.currentCloudHat === enums_1.CloudHat.Unknown) {
            this.currentCloudHat = enums_1.CloudHat.Clear;
            ClearCloudHat();
            return;
        }
        SetCloudHatTransition(this.cloudHatDict.has(this.currentCloudHat)
            ? this.cloudHatDict.get(this.currentCloudHat)
            : '', 3);
    }
    static get CloudHatOpacity() {
        return GetCloudHatOpacity();
    }
    static set CloudHatOpacity(value) {
        SetCloudHatOpacity(utils_1.clamp(value, 0, 1));
    }
    static get Weather() {
        switch (GetPrevWeatherTypeHashName()) {
            case -1750463879:
                return enums_1.Weather.ExtraSunny;
            case 916995460:
                return enums_1.Weather.Clear;
            case -1530260698:
                return enums_1.Weather.Neutral;
            case 282916021:
                return enums_1.Weather.Smog;
            case -1368164796:
                return enums_1.Weather.Foggy;
            case 821931868:
                return enums_1.Weather.Clouds;
            case -1148613331:
                return enums_1.Weather.Overcast;
            case 1840358669:
                return enums_1.Weather.Clearing;
            case 1420204096:
                return enums_1.Weather.Raining;
            case -1233681761:
                return enums_1.Weather.ThunderStorm;
            case 669657108:
                return enums_1.Weather.Blizzard;
            case -273223690:
                return enums_1.Weather.Snowing;
            case 603685163:
                return enums_1.Weather.Snowlight;
            case -1429616491:
                return enums_1.Weather.Christmas;
            case -921030142:
                return enums_1.Weather.Halloween;
            default:
                return enums_1.Weather.Unknown;
        }
    }
    static set Weather(value) {
        if (value !== enums_1.Weather.Unknown) {
            const weather = this.weatherDict[value];
            SetWeatherTypeOverTime(weather, 1);
            setTimeout(() => {
                SetWeatherTypeNow(weather);
                // tslint:disable-next-line: align
            }, 100);
        }
    }
    static get NextWeather() {
        switch (GetNextWeatherTypeHashName()) {
            case -1750463879:
                return enums_1.Weather.ExtraSunny;
            case 916995460:
                return enums_1.Weather.Clear;
            case -1530260698:
                return enums_1.Weather.Neutral;
            case 282916021:
                return enums_1.Weather.Smog;
            case -1368164796:
                return enums_1.Weather.Foggy;
            case 821931868:
                return enums_1.Weather.Clouds;
            case -1148613331:
                return enums_1.Weather.Overcast;
            case 1840358669:
                return enums_1.Weather.Clearing;
            case 1420204096:
                return enums_1.Weather.Raining;
            case -1233681761:
                return enums_1.Weather.ThunderStorm;
            case 669657108:
                return enums_1.Weather.Blizzard;
            case -273223690:
                return enums_1.Weather.Snowing;
            case 603685163:
                return enums_1.Weather.Snowlight;
            case -1429616491:
                return enums_1.Weather.Christmas;
            case -921030142:
                return enums_1.Weather.Halloween;
            default:
                return enums_1.Weather.Unknown;
        }
    }
    static set NextWeather(value) {
        if (value !== enums_1.Weather.Unknown) {
            const weather = this.weatherDict[value];
            SetWeatherTypeOverTime(weather, 0);
        }
    }
    /**
     * Doesn't work
     */
    static get WeatherTransition() {
        const transition = GetWeatherTypeTransition();
        return [this.weatherDict[transition[0]], this.weatherDict[transition[1]], transition[2]];
    }
    /**
     * Doesn't work
     */
    static set WeatherTransition(transition) {
        SetWeatherTypeTransition(transition[0], transition[1], transition[2]);
    }
    static transitionToWeather(weather, duration) {
        if (weather !== enums_1.Weather.Unknown) {
            SetWeatherTypeOverTime(this.weatherDict[weather], duration);
        }
    }
    static destroyAllCameras() {
        DestroyAllCams(false);
    }
    /**
     * Creates a blip at a given position and optionally radius.
     *
     * @param position World coordinate of blip.
     * @param radius (Optional) Radius of where blip should be shown.
     * @returns A Blip object.
     */
    static createBlip(position, radius) {
        if (radius !== null) {
            return new Blip_1.Blip(AddBlipForRadius(position.x, position.y, position.z, radius));
        }
        return new Blip_1.Blip(AddBlipForCoord(position.x, position.y, position.z));
    }
    static createCamera(position, rotation, fieldOfView) {
        return new Camera_1.Camera(CreateCamWithParams('DEFAULT_SCRIPTED_CAMERA', position.x, position.y, position.z, rotation.x, rotation.y, rotation.z, fieldOfView, true, 2));
    }
    /**
     * Create a ped at a desired location.
     *
     * @param model Ped model to be spawned.
     * @param position World position (coordinates) of Ped spawn.
     * @param heading Heading of Ped when spawning.
     */
    static createPed(model, position, heading = 0) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!model.IsPed || !(yield model.request(1000))) {
                return null;
            }
            return new models_1.Ped(CreatePed(26, model.Hash, position.x, position.y, position.z, heading, true, false));
        });
    }
    static createRandomPed(position) {
        return new models_1.Ped(CreateRandomPed(position.x, position.y, position.z));
    }
    /**
     * Create a vehicle at a desired location.
     *
     * @param model Vehicle model to be spawned.
     * @param position World position (coordinates) of Vehicle spawn.
     * @param heading Heading of Vehicle when spawning.
     * @returns Vehicle object.
     */
    static createVehicle(model, position, heading = 0) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!model.IsVehicle || !(yield model.request(1000))) {
                return null;
            }
            return new models_1.Vehicle(CreateVehicle(model.Hash, position.x, position.y, position.z, heading, true, false));
        });
    }
    /**
     * Create a random vehicle at a desired location.
     *
     * @param position World position (coordinates) of Vehicle spawn.
     * @param heading Heading of Vehicle when spawning.
     * @returns Vehicle object.
     */
    static createRandomVehicle(position, heading = 0) {
        return __awaiter(this, void 0, void 0, function* () {
            const vehicleCount = Object.keys(hashes_1.VehicleHash).length / 2; // check
            const randomIndex = utils_1.getRandomInt(0, vehicleCount);
            const randomVehicleName = hashes_1.VehicleHash[randomIndex];
            const modelHash = GetHashKey(randomVehicleName);
            const model = new _1.Model(modelHash);
            if (!model.IsVehicle || !(yield model.request(1000))) {
                return null;
            }
            return new models_1.Vehicle(CreateVehicle(model.Hash, position.x, position.y, position.z, heading, true, false));
        });
    }
    /**
     * Draw a marker at a desired location.
     *
     * @param type Type of marker.
     * @param position Location of marker.
     * @param direction Direction facing.
     * @param rotation World rotation.
     * @param scale Size of marker.
     * @param color Color of marker.
     * @param bobUpAndDown Animated movement along marker's own X axis.
     * @param faceCamera Rendering marker facing rendering camera.
     * @param rotateY Rotate along Y axis.
     * @param textureDict Custom texture dictionary for custom marker.
     * @param textureName Custom texture name for custom marker.
     * @param drawOnEntity Render the marker on an entity.
     */
    static drawMarker(type, position, direction, rotation, scale, color, bobUpAndDown = false, faceCamera = false, rotateY = false, textureDict = null, textureName = null, drawOnEntity = false) {
        DrawMarker(Number(type), position.x, position.y, position.z, direction.x, direction.y, direction.z, rotation.x, rotation.y, rotation.z, scale.x, scale.y, scale.z, color.r, color.g, color.b, color.a, bobUpAndDown, faceCamera, 2, rotateY, textureDict, textureName, drawOnEntity);
    }
    static drawLightWithRange(pos, color, range, intensity) {
        DrawLightWithRange(pos.x, pos.y, pos.z, color.r, color.g, color.b, range, intensity);
    }
    static drawSpotLight(pos, dir, color, distance, brightness, roundness, radius, fadeOut) {
        DrawSpotLight(pos.x, pos.y, pos.z, dir.x, dir.y, dir.z, color.r, color.g, color.b, distance, brightness, roundness, radius, fadeOut);
    }
    static drawSpotLightWithShadow(pos, dir, color, distance, brightness, roundness, radius, fadeOut) {
        DrawSpotLightWithShadow(pos.x, pos.y, pos.z, dir.x, dir.y, dir.z, color.r, color.g, color.b, distance, brightness, roundness, radius, fadeOut, 0);
    }
    static drawLine(start, end, color) {
        DrawLine(start.x, start.y, start.z, end.x, end.y, end.z, color.r, color.g, color.b, color.a);
    }
    static drawPoly(vertexA, vertexB, vertexC, color) {
        DrawPoly(vertexA.x, vertexA.y, vertexA.z, vertexB.x, vertexB.y, vertexB.z, vertexC.x, vertexC.y, vertexC.z, color.r, color.g, color.b, color.a);
    }
    static raycast(source, direction, maxDistance, options, ignoreEntity) {
        const target = utils_1.Vector3.add(source, utils_1.Vector3.multiply(direction, maxDistance));
        return new Raycast_1.RaycastResult(StartShapeTestRay(source.x, source.y, source.z, target.x, target.y, target.z, Number(options), ignoreEntity.Handle, 7));
    }
}
World.currentCloudHat = enums_1.CloudHat.Clear;
World.cloudHatDict = new Map([
    [enums_1.CloudHat.Unknown, 'Unknown'],
    [enums_1.CloudHat.Altostratus, 'altostratus'],
    [enums_1.CloudHat.Cirrus, 'Cirrus'],
    [enums_1.CloudHat.Cirrocumulus, 'cirrocumulus'],
    [enums_1.CloudHat.Clear, 'Clear 01'],
    [enums_1.CloudHat.Cloudy, 'Cloudy 01'],
    [enums_1.CloudHat.Contrails, 'Contrails'],
    [enums_1.CloudHat.Horizon, 'Horizon'],
    [enums_1.CloudHat.HorizonBand1, 'horizonband1'],
    [enums_1.CloudHat.HorizonBand2, 'horizonband2'],
    [enums_1.CloudHat.HorizonBand3, 'horizonband3'],
    [enums_1.CloudHat.Horsey, 'horsey'],
    [enums_1.CloudHat.Nimbus, 'Nimbus'],
    [enums_1.CloudHat.Puffs, 'Puffs'],
    [enums_1.CloudHat.Rain, 'RAIN'],
    [enums_1.CloudHat.Snowy, 'Snowy 01'],
    [enums_1.CloudHat.Stormy, 'Stormy 01'],
    [enums_1.CloudHat.Stratoscumulus, 'stratoscumulus'],
    [enums_1.CloudHat.Stripey, 'Stripey'],
    [enums_1.CloudHat.Shower, 'shower'],
    [enums_1.CloudHat.Wispy, 'Wispy'],
]);
World.weatherDict = [
    'EXTRASUNNY',
    'CLEAR',
    'CLOUDS',
    'SMOG',
    'FOGGY',
    'OVERCAST',
    'RAIN',
    'THUNDER',
    'CLEARING',
    'NEUTRAL',
    'SNOW',
    'BLIZZARD',
    'SNOWLIGHT',
    'XMAS',
];
exports.World = World;
